# -*- coding: utf-8 -*-
import logging
from twisted.python import log

def dbg(*a, **kw):
    kw["logLevel"] = logging.DEBUG
    log.msg(*a, **kw)

class CameraProtocol():
    def setSize(self, size):
        raise NotImplementedError()

    def getSize(self):
        raise NotImplementedError()

    def disconnect(self):
        dbg("CameraProtocol.disconnect not implemented")

    def updateSettings(self):
        dbg("CameraProtocol.updateSettings not implemented")

class Listener():
    def lostConnection(self, reason, failed, *a, **kw):
        raise NotImplementedError()

    def gotFrame(self, frame, *a, **kw):
        raise NotImplementedError()

    def forcedDisconnect(self, *a, **kw):
        dbg("Listener.forcedDisconnect not implemented")

class UnknownDevice(Exception):

    def __init__(self, address):
        self.address = address

    def __str__(self):
        return "Device unknown %s" % self.address



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time, wave, struct, array

from twisted.web import http, server
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.internet import defer
from twisted.internet import task
from twisted.web.resource import Resource
from twisted.python import log
from logging import DEBUG
from airi.camera import airicamera
from airi.camera.protocol import CameraFactory
from airi.camera import Listener
from airi import report, RESULT
from functools import partial
import twisted_bluetooth

HTTP_DISCONNECT_TIMEOUT = 10
CATEGORY = "AIRi-SCO"

class SCOStream():
    clients = []
    request = None

    @report(category=CATEGORY)
    def __init__(self, request):
        self.request = request
        self.request.connectionLost = self.connectionLost
        self.request.multipart=self
        SCOStream.clients.append(self)

    @report(category=CATEGORY)
    def process(self):
        self.request.setHeader('Cache-Control', 
                'no-cache, no-store, must-revalidate;')
        self.request.setHeader('Expires', '0');
        self.request.setHeader('Pragma-directive', 'no-cache')
        self.request.setHeader('Pragma', 'no-cache')
        self.request.setHeader('Cache-directive', 'no-cache')
        self.request.setHeader('Content-Type', 'audio/x-wav')

        # based on wave.py from the batteries
        self.request.write("RIFF")
        initlength = 0x7fffffdb
        nchannels = 1
        sampwidth = 2
        framerate = 8000
        nframes = initlength / (nchannels * sampwidth)
        datalength = nframes * nchannels * sampwidth
        self.request.write(struct.pack('<l4s4slhhllhh4s',
            36 + datalength, 'WAVE', 'fmt ', 16,
            wave.WAVE_FORMAT_PCM, nchannels, framerate,
            nchannels * framerate * sampwidth,
            nchannels * sampwidth,
            sampwidth*8, 'data'))
        self.request.write(struct.pack('<l', datalength))

    @report(category=CATEGORY)
    def connectionLost(self, reason):
        '''Called when the http stream is closed'''
        if self in SCOStream.clients:
            SCOStream.clients.remove(self)
        if len(SCOStream.getClients(self.target)):
            return
        log.msg("No one else is listening, killing link")
        twisted_bluetooth.SCODisconnect(self.target)
        del self
    
    @classmethod
    def sendToClients(klass, data, address):
        if len(klass.clients) == 0:
            return

        nframes = len(data) // (2 * 1) #sampwidth * channels
        data = array.array("h", data)
        data.byteswap()
        data = data.tostring()
        for client in klass.getClients(address):
            client.request.write(data)

    @classmethod
    def getClients(klass, address):
        def internal():
            for c in klass.clients:
                if c.target.lower() == address:
                    yield c
        address = address.lower()
        return list(internal())

class SCOResource(Resource, Listener):
    isLeaf = True
  
    @classmethod
    def getClients(klass, address):
        return SCOStream.getClients(address)

    def gotFrame(self, client):
        data = client.recv(4096)
        SCOStream.sendToClients(data, client.address)

    @report(category=CATEGORY)
    def lostConnection(self, client):
        '''Called when the Bluetooth Link is lost'''
        print "SCOResource.lostConnection", client.address
        clients = SCOStream.getClients(client.address)
        for c in clients:
            try:
                c.request.finish()
            except Exception, err:
                log.err(err)
            SCOStream.clients.remove(c)

    @report(category=CATEGORY, level=RESULT)
    def render_GET(self, request):
        request.transport.socket.settimeout(5)
        if not getattr(twisted_bluetooth, "SCOReader", None):
            raise Exception("Not supported in your platform")

        address = request.path.split("/",2)[-1].replace("_", ":")
        if address == "" or len(address) != 17:
            raise Exception("Invalid address")
        scoclient = SCOStream(request)
        scoclient.process()
        scoclient.target = address
        if len(address) == 17:
            if not twisted_bluetooth.SCOConnected(address):
                log.msg("Connecting SCO to %s" % address)
                client = twisted_bluetooth.SCOReader(address,
                        self.gotFrame, self.lostConnection)
                log.msg("SCO connected")
        return server.NOT_DONE_YET

if __name__ == '__main__':
    from twisted.web.server import Site
    import sys
    log.startLogging(sys.stdout)

    root = Resource()
    root.isLeaf = False
    root.putChild("sco", SCOResource())
    reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
    reactor.run()#!/usr/bin/env python




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from twisted.web import http, server
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.internet import defer
from twisted.internet import task
from twisted.web.resource import Resource
from twisted.python import log
from logging import DEBUG
from airi.camera.protocol import CameraFactory
from airi.camera import Listener
from airi import report, RESULT
from functools import partial

MULTIPARTRESPONSE= "--%s\r\nContent-Type: %s\r\nContent-Length: %s\r\n\r\n%s\r\n\r\n"
HTTP_DISCONNECT_TIMEOUT = 30 # half a minute timeout
CATEGORY = "AIRi-Stream"

class MultiPartStream():
    BOUNDARY = 'myBOUNDARY'
    clients = []
    request = None
    oneshot = False

    @report(category=CATEGORY)
    def __init__(self, request):
        self.request = request
        self.request.connectionLost = self.connectionLost
        self.request.multipart=self
        self.oneshot = self.request.args.get('oneshot', ['false',])[0].lower() == 'true'
        self.thumbnail = self.request.args.get('thumbnail', ['false',])[0].lower() == 'true'
        self.flash = 'flash' in self.request.args
        MultiPartStream.clients.append(self)

    @classmethod
    def getBoundary(klass):
        return "--%s\r\n" % (klass.BOUNDARY)

    def writeBoundary(self):
        self.request.write(MultiPartStream.getBoundary())

    def writeStop(self):
        self.request.write("%s--\r\n" % MultiPartStream.getBoundary())

    @report(category=CATEGORY)
    def process(self):
        self.request.setHeader('Cache-Control', 'no-cache, no-store, must-revalidate;')
        self.request.setHeader('Expires', '0');
        self.request.setHeader('Pragma-directive', 'no-cache')
        self.request.setHeader('Pragma', 'no-cache')
        self.request.setHeader('Cache-directive', 'no-cache')
        if not (self.oneshot or self.thumbnail):
            self.request.setHeader('Connection', 'Keep-Alive')
            if self.flash:
                #self.request.setHeader('Content-Type', 
                #   'multipart/x-mixed-replace')
                self.request.setHeader('Content-Type',
                    'text/html;')
            else:
                self.request.setHeader('Content-Type', 
                    'multipart/x-mixed-replace;boundary=%s' % (self.BOUNDARY))

    @report(category=CATEGORY)
    def connectionTimeout(self):
        '''Called after X seconds after the http stream is closed'''
        if len(MultiPartStream.getClients(self.target)):
            return
        log.msg("No more clients, closing link")
        CameraFactory.disconnect(self.target)
        del self
  
    @report(category=CATEGORY)
    def connectionLost(self, reason):
        '''Called when the http stream is closed'''
        if self in MultiPartStream.clients:
            MultiPartStream.clients.remove(self)
        log.msg("Registering to disconnect in %s seconds" % HTTP_DISCONNECT_TIMEOUT)
        reactor.callLater(HTTP_DISCONNECT_TIMEOUT, 
            MultiPartStream.connectionTimeout, self)
        StreamResource.tellClientCount(self.target)

    @report(category=CATEGORY)
    def finish(self):
        '''Gets called by oneshot or thumbnail modes''' 
        try:
            self.request.finish()
        except Exception, err:
            log.err(err)
        MultiPartStream.clients.remove(self)

    def sendPart(self, content, mime="text/html", MULTIPART=None):
        if self.oneshot or self.thumbnail:
            try:
                self.request.setHeader("Content-Type", mime)
                self.request.setHeader("Content-Size", len(content))
                self.request.write(content)
                self.request.write("\n\n")
                self.finish()
            except Exception, err:
                log.msg("error during sendPart")
                log.err(err)
        else:
            if MULTIPART is None:
                MULTIPART=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, mime, len(content), content)
            try:
                self.request.write(MULTIPART)
            except Exception, err:
                log.msg("error during sendPart")
                log.err(err)

    @classmethod
    def sendToClients(klass, content, mime="text/html"):
        if len(klass.clients) == 0:
            return

        size = len(content)
        out=MULTIPARTRESPONSE % (klass.BOUNDARY, mime, size, content)

        for client in klass.clients:
            client.sendPart(content, mime, out)

#       log.msg("sendToClients, size: %s, count: %s" %(len(content), len(klass.clients)), 
#           loglevel=DEBUG,
#           category="MultiPartStream")

    @classmethod
    def getClients(klass, address):
        def internal():
            for c in klass.clients:
                if c.target.lower() == address:
                    yield c
        address = address.lower()
        return list(internal())

class StreamResource(Resource, Listener):
    isLeaf = True
  
    @classmethod
    def getClients(klass, address):
        return MultiPartStream.getClients(address)
 
    @classmethod
    @report(category=CATEGORY)
    def tellClientCount(klass, address):
        from airi.api import UpdateManager
        UpdateManager.propagate(address, {
            "client_count": len(klass.getClients(address))})

    def gotFrame(self, frame, address):
        size = len(frame)
        out=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, "image/jpeg", size, frame)

        for client in StreamResource.getClients(address):
            client.sendPart(frame, "image/jpeg")

    @report(category=CATEGORY)
    def lostConnection(self, reason, failed, address):
        '''Called when the Bluetooth Link is lost'''
        print "StreamResource.lostConnection", address
        clients = StreamResource.getClients(address)
        for c in clients:
            c.sendPart(str(reason))
            if not c.oneshot:
                try:
                    c.request.finish()
                except Exception, err:
                    log.err(err)
            MultiPartStream.clients.remove(c)
        StreamResource.tellClientCount(address)

    @report(category=CATEGORY, level=RESULT)
    def render_GET(self, request):
        address = request.path.split("/",2)[-1].replace("_", ":")
        request.transport.socket.settimeout(5)
        multipart = MultiPartStream(request)
        multipart.process()
        multipart.target = address
        if len(address) == 17:
            ready = CameraFactory.isConnected(address) or CameraFactory.isPending(address)
            if multipart.thumbnail and not ready:
                multipart.sendPart("")
                return server.NOT_DONE_YET
            if not ready:
                method = CameraFactory.getCamera(address).get("transport", "rfcomm")
                method = request.args.get("method", [method,])[-1].upper()
                try:
                    CameraFactory.connect(address, method=method)
                except Exception, err:
                    log.msg("Failed while trying to connect")
                    log.err(err)
        CameraFactory.registerListener(address, self)
        StreamResource.tellClientCount(address)
        return server.NOT_DONE_YET

class TestStreamResource(Resource):
    isLeaf = True

    def render_GET(self, request):
        log.msg(str(request))
        log.msg(str(request.requestHeaders))
        multipart = MultiPartStream(request)
        multipart.process()
        return server.NOT_DONE_YET

    def render_POST(self, request):
        return self.render_GET(request)

if __name__ == '__main__':
    import webcam
    from twisted.web.server import Site
    import sys
    log.startLogging(sys.stdout)

    webcam.init(False)
    def sendFrame():
        MultiPartStream.sendToClients(webcam.repeat().tostring(), "image/jpeg")
        reactor.callLater(1/15., sendFrame)

    root = Resource()
    root.isLeaf = False
    root.putChild("stream", TestStreamResource())
    reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
    reactor.callLater(0, sendFrame)
    reactor.run()#!/usr/bin/env python




from twisted.web.resource import Resource
from twisted.web import http, server
from twisted.python import log
from twisted.internet import threads, reactor
from time import time
import json
import twisted_bluetooth as bluetooth
import traceback
from airi.camera.protocol import CameraFactory
from airi.settings import getSettings
from airi import report

settings = getSettings()

SCAN_TIMEOUT=60 # scan is started once each 60 seconds

class ConnectionTest():
    def __init__(self, req):
        self.request = req

    def cleanup(self, address):
        CameraFactory.disconnect(address)
        CameraFactory.removeListener(address, self)
        self.request.finish()

    def lostConnection(self, reason, failed, address):
        print "ConnectionTest.lostConnection", address
        self.request.write("<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % reason)
        self.request.setResponseCode(500, str(failed))
        self.request.finish()
        self.cleanup(address)

    def gotFrame(self, frame, address):
        print "ConnectionTest.gotFrame", address
        cli = CameraFactory.getCamera(address)
        self.request.write(json.dumps(cli))
        self.cleanup(address)

class DevicesManager(Resource):
    isLeaf = True

    def render_GET(self, request):
        request.setHeader('Content-Type', 'application/json')
        request.setHeader('Cache-Control', 'no-cache')
        request.setHeader('Connection', 'close')
        return json.dumps(list(CameraFactory.getCameras()))

class ConfigurationManager(Resource):
    isLeaf = True

    def render_GET(self, request):
        try:
            request.setHeader('Content-Type', 'application/json')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            if len(request.postpath) > 0 and len(request.postpath[0])>0:
                address=request.postpath[0].replace("_", ":")
                if len(request.args) == 0:
                    return json.dumps(CameraFactory.getCamera(address))
                camera = settings.getCamera(address)
                for key in request.args:
                        settings.setCameraSetting(address, key, request.args[key][-1])
                settings.save()
                cli = CameraFactory.getConnected(address)
                if cli:
                    cli.updateSettings()
                return json.dumps(CameraFactory.getCamera(address))
            request.setResponseCode(500, "invalid address")
            return "Invalid address"

        except Exception, err:
            print err
            CameraFactory.connect(address, 1, "RFCOMM")
            CameraFactory.registerListener(address, ConnectionTest(request))
            return server.NOT_DONE_YET

    def render_POST(self, request):
        out = {}
        print str(request.args)
        for key, value in request.args.iteritems():
            if len(value) > 1:
                out[key] = True
            else:
                out[key]=value[0]
        settings.setCamera(out)
        print out
        settings.save()
        cli = CameraFactory.getConnected(out['address'])
        if cli:
            cli.updateSettings()
        return "saved"

class doConfiguration(Resource):
    isLeaf = True

    def render_POST(self, request):
        address = request.args["address"][0]
        option = request.args["option"][0]
        value = request.args["value"][0]
        if value.lower() in ["true", "false"]:
            value = value.lower()=="true"
        log.msg("doConfiguration.POST(%s, %s, %s)" % (address, option, value))
        try:
            cli = CameraFactory.getConnected(address)
            if cli:
                cli.client.set(option, value)
            return "updated"
        except Exception, err:
            return "failed: %s" % err

class ConnectionManager(Resource):
    isLeaf = True

    def render_all(self, request):
        out = {}
        for addr, cli in CameraFactory.getConnected().iteritems():
            out[addr] = CameraFactory.getCamera(addr)
        return out

    def render_GET(self, request):
        try:
            request.setHeader('Content-Type', 'application/json')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            if len(request.postpath) > 0 and len(request.postpath[0])>0:
                address=request.postpath[0].replace("_", ":")
                cli = CameraFactory.getCamera(address, silent=True)
                if cli or not request.args.get("test", None):
                    return json.dumps(CameraFactory.getCamera(address))
                CameraFactory.connect(address, 1, "RFCOMM")
                CameraFactory.registerListener(address, ConnectionTest(request))
                return server.NOT_DONE_YET
            else:
                return json.dumps(self.render_all(request))
        except Exception, err:
            request.setHeader('Content-Type', 'text/html')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            request.setResponseCode(500, str(err))
            return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class ScanManager(Resource):
    isLeaf = True
    lastscan = None
    scancache = None
    scanning = None
    waiting = []

    def do_work(self):
        error = None
        try:
            log.msg("Doing scan")
            cache = bluetooth.discover_devices(lookup_names=True)
            log.msg(cache)
            self.scancache = [{
                    'address':x[0], 
                    'name':x[1], 
                    'state': CameraFactory.isConnected(x[0])}
                    for x in cache]
            log.msg(self.scancache)
            remove = []
            print self.waiting
            for request in self.waiting:
                request.setHeader('Content-Type', 'application/json')
                request.setHeader('Cache-Control', 'no-cache')
                request.setHeader('Connection', 'close')
                request.write(json.dumps(self.scancache))
                request.finish()
            if len(self.waiting)==0:
                return server.NOT_DONE_YET
        except Exception, err:
            log.err(err)
            error = err

        try:
            error = "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % traceback.format_exc()
            for request in self.waiting:
                if request.finished:
                    continue
                request.setHeader('Content-Type', 'text/html')
                request.setHeader('Cache-Control', 'no-cache')
                request.setHeader('Connection', 'close')
                request.setResponseCode(500, error)
                request.write("<html><h1>ERROR:</h1>\n")
                request.write("<pre>%s</pre></html>" % traceback.format_exc())
                request.finish()
        except Exception, err:
            log.err(err)
        self.waiting = []
        self.lastscan = None
        log.msg("Scan thread done")
        return server.NOT_DONE_YET


    def lost_connection(self, err, request):
        log.msg("lost_connection(%s)" % str(request))
        self.waiting.remove(request)
    
    def render_GET(self, request):
        if self.lastscan and time()-self.lastscan < SCAN_TIMEOUT:
            if not self.scancache:
                self.waiting.append(request)
                request.notifyFinish().addErrback(self.lost_connection, request)
                log.msg("%s" % self.waiting)
                log.msg("Tried to start a parallel scan, we wait")
                return server.NOT_DONE_YET

            request.setHeader('Content-Type', 'application/json')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            request.write(json.dumps(self.scancache))
            request.finish()
            log.msg("Scan return from cache")
            return server.NOT_DONE_YET

        self.lastscan = time()
        self.waiting.append(request)
        request.notifyFinish().addErrback(self.lost_connection, request)
        log.msg("%s" % self.waiting)
        log.msg("Starting scan thread")
        threads.deferToThread(self.do_work)
        return server.NOT_DONE_YET

class DisconnectManager(Resource):
    isLeaf = True

    def render_GET(self, request):
        try:
            request.setHeader('Content-Type', 'application/json')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            if len(request.postpath) > 0 and len(request.postpath[0])>0:
                address=request.postpath[0].replace("_", ":")
                if not CameraFactory.isConnected(address):
                    return json.dumps({"result": "Not Connected"})
                CameraFactory.disconnect(address)
                return json.dumps({"result": "Disconnected"})
            return json.dumps({"result": "No Address"})
        except Exception, err:
            request.setHeader('Content-Type', 'text/html')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            request.setResponseCode(500, str(err))
            return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class StateManager(Resource):
    isLeaf = True

    def render_POST(self, request):
        try:
            request.setHeader('Content-Type', 'application/json')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            address = request.args["address"][0]
            if CameraFactory.isConnected(address):
                CameraFactory.disconnect(address)
                return json.dumps({"result": "disconnected"})
            else:
                CameraFactory.connect(address)
                return json.dumps({"result": "connecting"})
        except Exception, err:
            request.setHeader('Content-Type', 'text/html')
            request.setHeader('Cache-Control', 'no-cache')
            request.setHeader('Connection', 'close')
            request.setResponseCode(500, str(err))
            return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class UpdateManager(Resource):
    isLeaf = True
    clients = {}
    reference = None

    @report(category="UpdateManager")
    def __init__(self):
        UpdateManager.reference=self

    @report("UpdateManager")
    def timeout(self, connection, address):
        log.msg("UpdateManager.timeout (%s, %s)" % ( connection, address ))
        connection.write(json.dumps({"result": "timeout"}))
        connection.finish()
        if address in UpdateManager.reference.clients:
            if connection in UpdateManager.reference.clients[address]:
                UpdateManager.reference.clients[address].remove(connection)

    @classmethod
    @report(category="UpdateManager")
    def propagate(klass, address, args):
        if not klass.reference:
            return

        if not address in klass.reference.clients:
            return

        if "address" not in args:
            args["address"] = address

        for r in klass.reference.clients[address]:
            try:
                r.write(json.dumps(args))
                r.finish()
                r.delayed.cancel()
            except Exception, err:
                log.err(err)
        klass.reference.clients.pop(address)

    @report(category="UpdateManager")
    def lost_connection(self, err, address, request):
        log.msg("updateManager.lost_connection(%s, %s)" % (address, request))
        request.delayed.cancel()
        if address in self.clients:
            if request in self.clients[address]:
                self.clients[address].remove(request)

    @report(category="UpdateManager")
    def render_POST(self, request):
        address = request.args["address"][0]
        if address not in self.clients:
            self.clients[address] = []
        self.clients[address].append(request)
        request.delayed = reactor.callLater(60, self.timeout, connection=request, address=address)
#        request.delayed.addErrback(log.err)
        request.notifyFinish().addErrback(self.lost_connection, address=address, request=request)
        request.setHeader('Content-Type', 'application/json')
        request.setHeader('Cache-Control', 'no-cache')
        request.setHeader('Connection', 'close')
        return server.NOT_DONE_YET

class API_Root(Resource):
    isLeaf = True

    def render_GET(self, request):
        return "Invalid"

class API(Resource):
    isLeaf = False

    def __init__(self, *a, **kw):
        log.msg("API()")
        Resource.__init__(self, *a, **kw)
        self.putChild("", API_Root())
        self.putChild("scan", ScanManager())
        self.putChild("devices", DevicesManager())
        self.putChild("connected", ConnectionManager())
        self.putChild("configure", ConfigurationManager())
        self.putChild("doconfigure", doConfiguration())
        self.putChild("disconnect", DisconnectManager())
        self.putChild("updates", UpdateManager())
        self.putChild("switchstate", StateManager())

if __name__=='__main__':
    from twisted.application.service import Application
    from twisted.application.internet import TCPServer
    from twisted.web.server import Site
    from twisted.internet import reactor
    import sys
    log.startLogging(sys.stdout)

    root = Resource()
    root.putChild("api", API())
    reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
    reactor.run()#!/usr/bin/env python




# -*- coding: utf-8 -*-
try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
except:
        pass

from twisted.python import log
import logging

ENTRY       = 10
ARGS        = 20
RESULT  = 30

class report:
    """
    Decorator that prints information about function calls.
    Based on: http://paulbutler.org/archives/python-debugging-with-decorators/
    """

    def __init__(self, level=ENTRY, category=None):
        self.level=level
        self.category = category

    def __call__(self, fn):
        def wrap(*args, **kwargs):
            if self.level >= ARGS:
                fc="call to %s.%s (%s,%s)" % (
                    fn.__module__, fn.__name__,
                    ', '.join( [ repr(a) for a in args ] ),
                    ', '.join( ["%s = %s" % (a, repr(b)) for a,b in kwargs.items()] )
                )
                log.msg(fc, level=logging.DEBUG, category=self.category)
            elif self.level >= ENTRY:
                log.msg("call to %s.%s" % (fn.__module__, fn.__name__), category=self.category, level=logging.DEBUG)

            ret = fn(*args, **kwargs)
            if self.level >= RESULT:
                log.msg("%s returned %s" % (fc, ret), level=logging.DEBUG)
            return ret
        if self.level > 0:
            return wrap
        return fn

__version__ = ('1', '0', 'c9')
__version__ = '.'.join(__version__)

try:
    import camera
    from camera import dbg, CameraProtocol
    import settings
    import twisted_bluetooth
    import stream
    import server
    import api
except Exception, err:
    log.msg("Failed while importing base modules, ignore if running distutils")
    log.err(err)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from airi.jinja import main as jmain
from twisted.internet import reactor
from twisted.python import log
import sys

def browser_ready():
    if 'win' in sys.platform:
        return True
    from subprocess import Popen, PIPE
    try:
        p = Popen(["xset", "-q"], stdout=PIPE, stderr=PIPE)
        p.communicate()
        return p.returncode == 0
    except:
        return False

def startbrowser():
    import webbrowser
    print "Starting up browser"
    webbrowser.open("http://localhost:8000")

def main():
    import sys
    if sys.platform == "linux2":
        from pair import main
        reactor.callWhenRunning(main)
    log.startLogging(sys.stdout)
    nobrowser = "--nobrowser" in sys.argv
    if nobrowser:
        sys.argv.remove("--nobrowser")
    if len(sys.argv) > 1:
        jmain(int(sys.argv[1]))
    else:
        jmain()
    if browser_ready() and not nobrowser:
        import webbrowser
        reactor.callLater(1, startbrowser)
    reactor.run()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import cv

global capture, show
show = False
capture = None

def init(show_=False):
    global capture, show, camera_index
    if show_:
        cv.NamedWindow("w1", cv.CV_WINDOW_AUTOSIZE)
    show = show_
    camera_index = 0
    capture = cv.CaptureFromCAM(camera_index)

def repeat():
    global capture #declare as globals since we are assigning to them now
    global camera_index
    global show
    frame = cv.QueryFrame(capture)
    if show:
        cv.ShowImage("w1", frame)
        c = cv.WaitKey(50)
        if c>-1 and chr(c) =='q':
            print chr(c)
            return None
    return cv.EncodeImage(".jpeg", frame, [cv.CV_IMWRITE_JPEG_QUALITY, 50])

if __name__=='__main__':
    init(True)
    while repeat():
        continue




# -*- coding: utf-8 -*-
# Copyright (c) 2011 Naranjo Manuel Francisco <manuel@aircable.net>
# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
# See Twisted LICENSE for details.

"""
This class is used for Mac OSx Bluetooth connections.
"""

# System Imports
import os, types, sys, operator, socket
import lightblue
import socket

from multiprocessing.connection import Client

def main():
    pipe = Client(sys.argv[1])
    proto, target = pipe.recv()

    try:
        s = lightblue.socket(proto)
        s.connect(target)
        s.settimeout(1/100.)
        pipe.send(None)
        while True:
            if pipe.poll():
                b = pipe.recv()
                print "sending", b
                s.send(b)
            try:
                o = s.recv(4096)
                pipe.send(o)
            except Exception, err:
                pass

    except Exception, err:
        pipe.send(err)
        raise

if __name__=='__main__':
    if len(sys.argv) < 2:
        print "Usage %s <socket>" % sys.argv[0]
        sys.exit(1)
    main()




# -*- coding: utf-8 -*-
from twisted.web.resource import Resource
from twisted.python import log
from twisted.web.static import File
from jinja2 import Environment, PackageLoader
from airi import __version__
from airi.camera.protocol import CameraFactory
from airi.camera import UnknownDevice
from airi.settings import getSettings
from airi.twisted_bluetooth import resolve_name
from airi.stream import StreamResource
from airi.sco import SCOResource
import airi.twisted_bluetooth as bluetooth
import pkg_resources, os, time

settings = getSettings()

class TemplateResource():
    def __init__(self, template, context):
        self.template = template
        self.context = context

    def render(self, *args, **kwargs):
        return str(self.template.render(self.context))

class Main(Resource):
    isLeaf = False

    def __init__(self, home="/"):
        Resource.__init__(self)
        self.env = Environment(loader=PackageLoader("airi", "templates"))
        self.env.globals["home"]=home

    def index(self, request):
        devices = list(CameraFactory.getCameras())
        for dev in devices:
            dev["viewers"] = StreamResource.getClients(dev["address"])
        return {
          "devices": devices 
        }

    def setup(self, request):
        def saveCamera(args):
            if "enable_pincode" in args:
                if args["enable_pincode"][0].lower()=="true":
                    settings.setPIN(block=args["address"][0],
                          npin=args["pincode"][0])
                else:
                    settings.delPIN(args["address"][0])
            settings.save()
            for k in ["save", "last", "battery", "status",
                            "enable_pincode", "pincode", "new-device"]:
                if k in args:
                    args.pop(k)
            for k in args.keys():
                args[k]=args[k][0]
            print "saving camera", args
            settings.setCamera(args)
            settings.save()

        def deleteCamera(address):
            settings.deleteCamera(address)
            settings.save()

        def getCamera(address):
            out = {}
            try:
                out.update(CameraFactory.getCamera(address))
            except UnknownDevice, err:
                out["address"] = address
                out["name"] = resolve_name(address)
                out["new_device"] = True
                out["types"] = CameraFactory.getTypes()
            return out

        if "address" not in request.args:
            raise Exception("Invalid address")

        address = request.args["address"][-1]

        if request.method == "POST":
            if "save" in request.args:
                saveCamera(request.args)
            if "delete" in request.args:
                deleteCamera(address)
        return getCamera(address)

    def scan(self, request):
        try:
            log.msg("Doing scan")
            cache = bluetooth.discover_devices(lookup_names=True)
            log.msg(cache)
            out = {"devices":
                    [{
                      'address':x[0],
                      'name':x[1],
                      'state': CameraFactory.isConnected(x[0])
                    }
                        for x in cache
                    ]
                }
            log.msg(out)
            return out
        except Exception, err:
            log.err(err)
            return {"error": str(err)}

    def server_setup(self, request):
        if request.method == "POST":
            if "delete" in request.args:
                settings.delPIN(request.args["delete"][0])
                settings.save()
            elif "save" in request.args:
                request.args.pop("save")
                blocks = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("block") ])
                pins = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("value") ])
                sets = {}
                for b in blocks:
                    if len(blocks[b].strip()) == 0:
                        continue
                    settings.setPIN(block=blocks[b], npin=pins["value"+b.replace("block","")])
                settings.save()

        out = {}
        out["pins"]=settings.getPINs()
        return out

    def stream(self, request):
        if "address" not in request.args:
            raise Exception("You need to provide with an address")
        camera = CameraFactory.getCamera(request.args["address"][0], True)
        if camera["enable_pincode"]:
            camera["pincode"]="************"
        return {
          "isChrome": "chrome" in request.requestHeaders.getRawHeaders("user-agent")[0].lower(),
          "address": request.args["address"][0],
          "camera": camera
        }

    contexts = {
        "index.html": index,
        "setup.html": setup,
        "scan.html": scan,
        "server-setup.html": server_setup,
        "stream.html": stream
    }

    def getChild(self, path, request):
        if "media" in path or "favicon.ico" in path:
            return Resource.getChild(self, path, request)
        if len(path) == 0:
            path = "redirect.html"
        template = self.env.get_template(path)
        context = self.contexts.get(path, lambda x,y: {})(self, request)
        context["pairing_supported"]=bluetooth.isPairingSupported()
        context["l2cap_supported"]=bluetooth.L2CAP_SUPPORTED
        context["sco_supported"]=bluetooth.SCO_SUPPORTED
        context["version"] = __version__
        return TemplateResource(template, context)

def main(port=8000):
    from twisted.application.service import Application
    from twisted.application.internet import TCPServer
    from twisted.web.server import Site
    from twisted.internet import reactor
    from airi.api import API
    import sys
    log.startLogging(sys.stdout)

    MEDIA = pkg_resources.resource_filename("airi", "/media")
    FAVICON = pkg_resources.resource_filename("airi", "/media/favicon.ico")
    print "serving static content from", MEDIA

    root = Main()
    path = os.path.dirname(os.path.realpath(__file__))
    root.putChild("api",        API())
    root.putChild("media", File( MEDIA, defaultType=None ) )
    root.putChild("sco", SCOResource())
    root.putChild("stream", StreamResource())
    root.putChild("favicon.ico", File( FAVICON, defaultType=None) )
    p=reactor.listenTCP(port, Site(root), interface="0.0.0.0", backlog=5)
    return p

if __name__=='__main__':
    from twisted.internet import reactor
    main()
    reactor.run()




#!/bin/python

'''
AIRi install script
'''
import os, sys, subprocess

def downloadVirtualEnv():
    print "downloading virtualenv"
    import urllib2
    r = urllib2.urlopen("https://raw.github.com/pypa/virtualenv/develop/virtualenv.py")
    b=r.read()
    r.close()
    A=open("virtualenv.py", "wb")
    A.write(b)
    A.close()

try:
    import virtualenv
except:
    downloadVirtualEnv()
    import virtualenv

def getDefaultInstallationPath():
    def internalInstallationPath():
        if sys.platform.startswith("linux"):
            if os.access("/opt", os.W_OK):
                return "/opt"
            return os.path.expanduser("~")
        elif sys.platform.startswith("darwin"):
            if os.access("/Applications", os.W_OK):
                return "/Applications"
            return os.path.expanduser("~")
        elif sys.platform.startswith("win"):
            if os.access(os.environ["PROGRAMFILES"], os.W_OK):
                return os.environ["PROGRAMFILES"]
            return os.environ["APPDATA"]
        raise RuntimeError("Platform not supported")
    return os.path.join(internalInstallationPath(), "AIRi")

def getInstallationPath():
    default = getDefaultInstallationPath()
    r = raw_input("Do you want to install AIRi on %s [Y/n]? " % default)
    if r.lower().strip().startswith("y") or len(r.strip())==0:
        return default
    p = raw_input("Please type in install path: ").strip()
    if len(p) == 0:
        raise RuntimeError("Path can't be empty!")
    return p

def dohelp():
    print "Usage: %s --path path [install|update|remove]" % sys.argv[0]
    sys.exit(0)

def install(bin_):
    print "Creating virtualenv"
    virtualenv.main()
    print "Installing AIRi with dependencies"
    subprocess.call([os.path.join(bin_, "pip"), "install", "AIRi"])

def upgrade(bin_):
    print "Upgrading AIRi"
    subprocess.call([os.path.join(bin_, "easy_install"), "-U", "AIRi"])

def main():
    path = None
    method = None
    i = 0
    if "--path" in sys.argv:
        path = sys.argv[sys.argv.index("--path")+1]
    if "--help" in sys.argv:
        dohelp()

    method = sys.argv[-1].lower().strip()
    if method not in ["install", "update", "remove"]:
        dohelp()

    if method == "install":
        if not path:
            try:
                path = getInstallationPath()
            except KeyboardInterrupt:
                sys.exit(1)
    else:
        if not hasattr(sys, "real_prefix"):
            print "Need to be called from a virtualenv"
            sys.exit(0)
        path = sys.prefix
    print "Doing", method, "in", path

    if method=="remove":
        from twisted.python.filepath import FilePath
        FilePath(path).remove()
        sys.exit(0)

    sys.argv=sys.argv[:1]
    sys.argv.append(path)
    bin_ = virtualenv.path_locations(path)[-1]
    if method == "install":
        install(bin_)
    if not os.path.isdir(os.path.join(path, bin_)):
        print "You need to run install first!"
        sys.exit(1)
    upgrade(bin_)
    print "Run AIRi:", os.path.join(path, bin_, "AIRi")
    print "Handle AIRi package:", os.path.join(path, bin_, "AIRi-setup")

if __name__=='__main__':
    main()



# -*- coding: utf-8 -*-
from twisted.application.internet import TCPServer
from twisted.web.resource import Resource
from twisted.web.server import Site
from twisted.web import server
from twisted.web.static import File
from twisted.internet import reactor
from twisted.python import log
from twisted.web.util import Redirect
from airi.api import API
from airi.stream import StreamResource, MultiPartStream
from functools import partial
import os

__all__=['root', 'site']
path = os.path.dirname(os.path.realpath(__file__))

root = Resource()
root.putChild("api",        API())
root.putChild("media",      File(os.path.join(path, "media/")))
root.putChild("",           File(os.path.join(path, "media/devices.html")))
root.putChild("scan",       File(os.path.join(path, "media/scan.html")))
root.putChild("configure",  File(os.path.join(path, "media/configure.html")))
root.putChild("devices",    File(os.path.join(path, "media/devices.html")))
root.putChild("video",      File(os.path.join(path, "media/video.html")))
root.putChild("stream",     StreamResource())
site = Site(root)

if __name__=='__main__':
    import sys
    log.startLogging(sys.stdout)
    reactor.listenTCP(8000, site, interface="0.0.0.0")
    reactor.run()



# -*- coding: utf-8 -*-
from twisted.python import log
from twisted.internet import reactor
from airi.camera import dbg, CameraProtocol
from airi.settings import getSettings


settings=getSettings()

SIZES={
        "VGA":  4,
        "QVGA": 5,
        "QQVGA":6,
        "SVGA": 0xc,
        "XVGA": 0xd,
}

CAPABILITIES={ 
    'size':             SIZES.keys(),
    'pan':              [],
    'flash':            False,
    'voice':            False,
    'exposure':     False,
    'transport':    ['RFCOMM',],
    'battery':      False,
}

IDLE=0
ECHO=1
PREVIEW=2
COMMAND_MODE=3

class OptiEye(CameraProtocol):
    client = None
    state = IDLE
    callLater = None
    size = "QVGA"

    def getType(self):
        return 'OptiEye'

    def getCapabilities(self):
        return CAPABILITIES

    def getTransport(self):
        return "RFCOMM"

    def __init__(self, client):
        self.client = client
        self.address = client.address
        self.transport = client.transport
        self.doCommandMode()

    def setSize(self, size):
        if size not in SIZES:
            raise RuntimeException("Size not valid")
        self.size = size

    def getSize(self):
        return self.size

    def getSizes(self):
        return SIZES

    def __doCommand(self, command):
        self.transport.write("$GENIESYS%04X\r\n" % command)

    def updateSettings(self):
        self.doCommandMode()
        
    def __cancelLater(self):
        if not self.callLater:
            return
        if self.callLater.active():
            self.callLater.cancel()
        self.callLater = None

    def doCommandMode(self):
        dbg("doCommandMode")
        self.state = COMMAND_MODE
        self.__doCommand(1)
        self.__cancelLater()
        self.callLater = reactor.callLater(1, self.doSetSize)
        #self.callLater.addErrback(log.err)

    def doSetSize(self, size=None):
        self.state = ECHO
        self.__cancelLater()
        if not size:
            camera=settings.getCamera(self.address)
            size = "QVGA" #default
            if camera:
                size = camera.get("size", size)
        dbg("doSetSize(%s)", size)
        self.__doCommand(SIZES[size])
        self.callLater = reactor.callLater(2, self.doPreview)
        #self.callLater.addErrback(log.err)

    def doPreview(self):
        dbg("doPreview")
        self.state = PREVIEW
        self.__cancelLater()
        self.callLater = reactor.callLater(20, self.transport.loseConnection)
        #self.callLater.addErrback(log.err)
        self.__doCommand(2)

    def previewData(self):
        #dbg("previewData")
        self.callLater.reset(20)
        start = self.client.buffer.find("\xff\xd8")
        end = self.client.buffer.find("\xff\xd9")
        if start == -1 or end == -1:
            return

        frame = self.client.buffer[start:end+2]
        self.client.buffer = self.client.buffer[end+2:]
        self.client.gotFrame(frame)

    def dataReceived(self):
        #dbg("OptiEye.dataReceived, have %s bytes" % len(self.client.buffer))
        #dbg("state = %s" % self.state)

        if self.state == COMMAND_MODE:
            self.callLater.reset(1) # wait another second
            #dbg("Dropping %s" % self.client.buffer)
            self.client.buffer = ""
            return

        if self.state == PREVIEW:
            return self.previewData()

    def set(self, option, value):
        dbg("set %s->%s" % (option, value) )
        if option=="size":
                self.doSetSize(value)
                from airi.api import UpdateManager
                UpdateManager.propagate(self.address, {"size": value})
        else:
                dbg("ignored")

    def disconnect(self):
        dbg("OptiEyes.disconnect")
        self.__doCommand(1)

OptiEye.Capabilities = CAPABILITIES
OptiEye.Sizes = SIZES
OptiEye.Panning = None



# -*- coding: utf-8 -*-
import ConfigParser
from os import path, access, W_OK, environ
from twisted.python import log
import logging, sys, os
from airi import report

DEFAULTS={
    "dongles": {
        "default": True,
    },
    "pin": {
        "default": 1234,
    }
}

def resolvehome():
    paths = [ path.expanduser("~") ]
    for i in ["HOME", "HOMEPATH", "DATA_PATH"]:
        if i in os.environ:
            print "Has", i
            paths.append(os.environ[i])
    for i in paths:
        if access(i, W_OK):
            return i

class Settings():
    instance = None

    def __init__(self, name=None):
        if not name:
            parent = resolvehome()
            if not parent:
                if access(path.dirname(path.realpath(__file__)), W_OK):
                    print "Using path.realpath"
                    parent = path.dirname(path.realpath(__file__))
                else:
                    raise RuntimeException("Can't find suitable path to store settings")
            if not sys.platform.startswith("win"):
                name=path.join(parent, ".AIRi")
            else:
                name=path.join(parent, "AIRi.cfg")
        print "configuration file", name
        self.name = name
        self.reload()


    def reload(self):
        self.config = ConfigParser.SafeConfigParser()
        for section in DEFAULTS:
            if self._has_section(section):
                continue;

            self._add_section(section)
            for key, value in DEFAULTS[section].iteritems():
                self._set(section, key, str(value))

        if path.exists(self.name) and path.isfile(self.name):
            self._read(self.name)

    # @report()
    def getDongle(self, address):
        self.reload()
        address = address.replace(":", "_")
        dongles = self._options("dongles")
        for op in dongles:
            if address.lower().startswith(op):
                return self._getboolean("dongles", op)
        return self._getboolean("dongles", "default")

    # @report()
    def getCameraSections(self):
        self.reload()
        cameras = self._sections()
        if "dongles" in cameras:
            cameras.remove("dongles")
        if "pin" in cameras:
            cameras.remove("pin")
        return cameras

    # @report()
    def getCameraSection(self, address, create=False):
        self.reload()
        cameras = self.getCameraSections()
        for cam in cameras:
            if address.lower().startswith(self._get(cam, "address").lower()):
                return cam
        if create:
            cameras=self._sections()
            if "dongles" in cameras:
                cameras.remove("dongles")
            section="camera%i"%(len(cameras)+1)
            self._add_section(section)
            self._set(section, "address", address)
            return section
        return None

    #@report(level=20)
    def __cameraDict(self, items):
        # defaults
        out = {
            "reconnect_timeout": 10,
            "reconnect": False,
            "voice": False,
            "flash": False,
            "exposure": 0,
            "pan": "none"
        }
        for key, val in items:
            if key in ['reconnect_timeout', 'exposure']:
                out[key] = int(val)
            elif key in ['reconnect', 'enable', 'voice', 'flash', "enable_pincode"]:
                out[key] = val.lower() == "true"
            else:
                out[key] = val
        try:
            out["pincode"] = self.getPIN(out["address"], perfect=True, silent=False)
            out["enable_pincode"] = True
        except Exception, err:
            out["enable_pincode"] = False
        if "type" not in out:
            return None
        return out

    #@report(debug=True)
    def getCameras(self):
        self.reload()
        for camera in self.getCameraSections():
            c = self.__cameraDict(self._items(camera))
            if c: yield c

    def deleteCamera(self, address):
        self.reload()
        cam = self.getCameraSection(address)
        if not cam:
            return
        self._remove_section(cam)

    #@report(debug=True)
    def getCamera(self, address):
        self.reload()
        cam = self.getCameraSection(address)
        if cam:
            return self.__cameraDict(self._items(cam))
        return None

    #@report(debug=True)
    def save(self):
        self._write(open(self.name, "wb"))

    #@report(level=30)
    def __sanitizeSetting(self, key, val):
        if key in ['reconnect_timeout', 'exposure']:
            val = int(val)
        elif key in ['reconnect', 'enable', 'voice', 'flash', "enable_pincode"]:
            if type(val) != bool:
                val = val.lower() in ["true", "ok"]
        return str(val)

    #@report(level=20)
    def setCameraSetting(self, address, key, value, section=None):
        if not section:
            section = self.getCameraSection(address, True)
        if not value:
            self._remove_option(section, key)
        else:
            self._set(section, key, self.__sanitizeSetting(key, value))

    #@report(level=20)
    def setCamera(self, configuration):
        if "address" not in configuration:
            raise Exception("You need to set address")

        section = self.getCameraSection(configuration["address"], True)

        for key, value in configuration.iteritems():
            self.setCameraSetting(configuration["address"], key, value, section=section)
        self.save()
        from airi.api import UpdateManager
        UpdateManager.propagate(configuration["address"], configuration)

    # @report(debug=True)
    def setDongle(self, block, enable=False):
        self._set("dongles", block.replace(":", "_"), str(enable))

    # @report(debug=True)
    def setPIN(self, npin, block="default"):
        self._set("pin", block.replace(":", "_"), str(npin))
        from airi.api import UpdateManager
        UpdateManager.propagate(block.replace(":", "_"), {"pin": npin})

    # @report(debug=True)
    def delPIN(self, block):
        self._remove_option("pin", block.replace(":", "_"))
        from airi.api import UpdateManager
        UpdateManager.propagate(block.replace(":", "_"), {"pin": None})

    # @report(debug=True)
    def getPINs(self):
        self.reload()
        if not self.config.has_section("pin"):
            pins = {"default": default}
        else:
            pins = dict((b[0].replace("_", ":"), b[1]) for b in self.config.items("pin"))
        return pins

    # @report(debug=True)
    def getPIN(self, address=None, perfect=False, default="1234", silent=True):
        self.reload()
        if not self.config.has_section("pin"):
            pins = {"default": default}
        else:
            pins = dict((b[0].lower(), b[1]) for b in self.config.items("pin"))
        if not address:
            log.msg("No address falling back to default", level=logging.DEBUG)
            return pins.get("default", default)
        address=address.replace(":", "_").lower()

        for block in pins:
            if not perfect:
                if address.startswith(block):
                    return pins[block]
            else:
                if address == block:
                    return pins[block]
        if not silent:
            raise Exception("PIN not provided")
        return pins.get("default", default)

    def __getattr__(self, name):
        return getattr(self.config, name[1:])

def getSettings(name=None):
    if Settings.instance:
        return Settings.instance
    Settings.instance = Settings(name)
    return Settings.instance

if __name__=='__main__':
    import logging
    logging.basicConfig(level=logging.DEBUG)
    logging.info("starting demo")
    s=Settings()
    logging.info(s.getDongle("00:25:BF:11:22:33"))
    s.setDongle("00:25:BF", "false")
    logging.info(s.getDongle("00:25:BF:11:22:33"))
    logging.info(s.getCamera("00:25:BF:11:22:33"))
    logging.info(s.getCamera("00:25:BF:00:25:BF"))
    s.setCamera({"address": "00:25:BF:00:25:AA", "reconnect": True})
    s.save()




# -*- coding: utf-8 -*-
from twisted.python import log
from twisted.internet import reactor
from airi.camera import dbg, CameraProtocol
from airi.settings import getSettings
from datetime import datetime
import airi.twisted_bluetooth as bluetooth

settings = getSettings()

SIZES = {
    #day modes
    "QVGA": 1,
    "VGA": 2,
    "XGA": 3,
    "QXGA": 5,
    "720P": 6,

    "QVGA Zoom 1": 11,
    "QVGA Zoom 2": 12,
    "QVGA Zoom 3": 13,
}

PANS = ["none", "right", "left", "up", "down"]

CAPABILITIES = {
    'size': [a[0] for a in sorted(SIZES.iteritems(), 
            key=lambda(x, y):(y, x))],
    'flash': True,
    'pan': PANS,
    'voice': False, # by now ;)
    'exposure': True,
    'transport': ['RFCOMM', 'L2CAP'],
    'battery': True,
}

COMMANDS = dict()


class Command():
    def __init__(self, name, timeout, command):
        self.name = name
        self.timeout = timeout
        self.command = command
        COMMANDS[name] = self

Command("date", 1, "D")
Command("flash", 1, "F")
Command("size", 3, "S")
Command("pan", 1, "P")
Command("exposure", 1, "E")
Command("stream", 1, "L")


class AIRi(CameraProtocol):
    client = None
    callLater = None
    size = "QVGA"
    defaults = {
        "size": "QVGA",
        "flash": 0,
        "voice": False,
        "exposure": 0,
        "stream": True,
        "pan": "none",
    }

    def getType(self):
        return 'AIRi'

    def __init__(self, client):
        self.client = client
        self.address = client.address
        self.transport = client.transport
        self.pending = []
        self.callLater = None
        self.sco_socket = None
        self.doSetup()
        self.buffer = ""

    def internalDoCommand(self):
        if len(self.pending) == 0:
            return
        command, value, timeout = self.pending.pop(0)
        if callable(value):
            value = value()
        dbg("doCommand $%s%s" % (command, value))
        self.transport.write("$%s%s\n\r" % (command, value))
        self.callLater = reactor.callLater(timeout, 
            self.internalDoCommand)

    def doCommand(self, command, value):
        if type(command) != Command:
            command = COMMANDS[command]
        self.pending.append((command.command, value, command.timeout))
        if self.callLater and self.callLater.active():
            return
        self.internalDoCommand()

    def doSetup(self):
        dbg("doSetup")
        nsets = settings.getCamera(self.address)
        sets = self.defaults
        size = self.size
        if nsets:
            sets.update(nsets)
        dbg(sets)
        self.callLater = reactor.callLater(1, self.internalDoCommand)
        self.doCommand("date",
                        lambda: datetime.utcnow().strftime("%Y%m%dT%H%M%SZ"))
        self.doCommand("size", SIZES[sets["size"]])
        self.doCommand("stream", "1" if sets["stream"] else "0")
        self.doCommand("flash", "0" if not sets["flash"] else "1")
        self.doCommand("pan", sets["pan"].upper()[0])

    def updateSettings(self):
        self.doSetup()

    def previewData(self):
        start = self.client.buffer.find("\xff\xd8")
        end = self.client.buffer.find("\xff\xd9")
        if start == -1 or end == -1:
            return

        frame = self.client.buffer[start:end + 2]
        self.client.buffer = self.client.buffer[end + 2:]
        self.client.gotFrame(frame)

    def SCO_Data(self, sock, lost=False):
        if lost:
            reactor.removeReader(self.sco_socket)
            return
        self.buffer += sock.recv(bluetooth.sco_mtu)
        if len(self.buffer) >= 400:
            from airi.stream import MultiPartStream
            MultiPartStream.sendToClients(self.buffer,
                mime="application/octet-stream")
            self.buffer = ""

    def enableSCO(self):
        if self.sco_socket != None:
            return

        self.sco_socket = bluetooth.SCOReader(self.address, self.SCO_Data)
        reactor.addReader(self.sco_socket)

    def disableSCO(self):
        if not self.sco_socket:
            return
        self.sco_socket.close()
        reactor.removeReader(self.sco_socket)
        self.sco_socket = None

    def doSCO(self, enable):
        if not getattr(bluetooth, "SCOReader", None):
            dbg("sco not supported in this platform")
            return
        if enable:
            return self.enableSCO()
        else:
            return self.disableSCO()

    def dataReceived(self):
        return self.previewData()

    def disconnect(self):
        dbg("AIRi.disconnect")
        self.disableSCO()

    def set(self, option, value):
        dbg("set %s-> %s" % (option, value))
        if option == "size":
            self.doCommand(option, SIZES[value])
        elif option == "flash" or option=="stream":
            self.doCommand(option, "0" if not value else "1")
        elif option == "pan" or option == "exposure":
            self.doCommand(option, value.upper())
        elif option == "voice":
            self.doSCO(value)
        else:
            dbg("ignored")

AIRi.Capabilities = CAPABILITIES
AIRi.Sizes = SIZES
AIRi.Panning = PANS




#!/usr/bin/python
# -*- coding: utf-8 -*-
from twisted.internet import reactor
from twisted.python import log

from functools import partial
import sys, os
import gobject, dbus, dbus.service
from dbus.exceptions import DBusException
from dbus.mainloop.glib import DBusGMainLoop

DBusGMainLoop(set_as_default=True)

PATH="/net/aircable/pairing"

from settings import getSettings
settings=getSettings()

def getPIN(address, dongle):
    return settings.getPIN(address)

def handle_name_owner_changed(own, old, new):
    if own.startswith('org.bluez'):
        if new is None or len(str(new))==0:
            log.msg( "bluez has gone down, time to get out")
        else:
            log.msg( "bluez started, time to restart")

def registerAgent(bus, path):
    try:
        adapter = dbus.Interface(bus.get_object("org.bluez", path),
            "org.bluez.Adapter")
        adapter.RegisterAgent(PATH, "KeyboardOnly")
        Agent.listeners.append(path)
        log.msg("adapter registered for path %s" % path)
    except DBusException, err:
        if "org.bluez.Error.AlreadyExists" == err.get_dbus_name():
            log.msg("Can't register pairing agent for %s" % path)
            Agent.bus_non_available=True
            return False
        log.err(err)
    return True

def handle_adapter_added(path, signal, bus):
    log.msg("bluez.%s: %s" % (signal, path))
    registerAgent(bus, path)

def handle_adapter_removed(path, signal):
    log.msg("adapter removed %s" % path)
    if path in Agent.listeners: Agent.listeners.pop(path)

class Rejected(dbus.DBusException):
    _dbus_error_name = "org.bluez.Error.Rejected"

class Agent(dbus.service.Object):
    listeners = list()
    exit_on_release = True
    instance = None

    def __init__(self, *args, **kwargs):
        dbus.service.Object.__init__(self, *args, **kwargs)
        self.bus = self._connection
        self.path = self._object_path

    def set_exit_on_release(self, exit_on_release):
        self.exit_on_release = exit_on_release

    def log(self, message):
        log.msg("%s: %s" % (self.path, message))

    @dbus.service.method("org.bluez.Agent",
            in_signature="", out_signature="")
    def Release(self):
        self.log("Agent Release")
        self.bus._unregister_object_path(self.path)
        if self.exit_on_release:
            self.log("Exiting from loop")
            reactor.stop()

    @dbus.service.method("org.bluez.Agent",
                  in_signature="os", out_signature="")
    def Authorize(self, device, uuid):
        self.log("Authorize (%s, %s)" % (device, uuid))

    @dbus.service.method("org.bluez.Agent",
                in_signature="o", out_signature="s")
    def RequestPinCode(self, path):
        self.log("RequestPinCode (%s)" % path)
        device = dbus.Interface(self.bus.get_object("org.bluez", path),
                "org.bluez.Device")
        dongle = dbus.Interface(self.bus.get_object("org.bluez",
            device.GetProperties()['Adapter']),
            "org.bluez.Adapter")
        device=str(device.GetProperties()['Address'])
        dongle=str(dongle.GetProperties()['Address'])
        pin=getPIN(device, dongle)
        self.log("RequestPinCode (%s->%s): %s" % (dongle, device, pin))
        return pin

    @dbus.service.method("org.bluez.Agent",
            in_signature="o", out_signature="u")
    def RequestPasskey(self, device):
        self.log("RequestPasskey (%s): %s" % (device, PIN) )
        return dbus.UInt32(PIN)

    @dbus.service.method("org.bluez.Agent",
            in_signature="ou", out_signature="")
    def DisplayPasskey(self, device, passkey):
        self.log("DisplayPasskey (%s, %d)" % (device, passkey))

    @dbus.service.method("org.bluez.Agent",
            in_signature="ou", out_signature="")
    def RequestConfirmation(self, device, passkey):
        self.log("RequestConfirmation (%s, %d)" % (device, passkey))
        device = dbus.Interface(self.bus.get_object("org.bluez", device),
                "org.bluez.Device")
        dongle = dbus.Interface(self.bus.get_object("org.bluez",
            device.GetProperties()['Adapter']),
            "org.bluez.Adapter")
        device=str(device.GetProperties()['Address'])
        dongle=str(dongle.GetProperties()['Address'])
        pin=getPIN(device, dongle)
        if passkey == pin:
            self.log("passkey matches")
            return
        self.log("passkey doesn't match")
        raise Rejected("Passkey doesn't match")

    @dbus.service.method("org.bluez.Agent",
            in_signature="s", out_signature="")
    def ConfirmModeChange(self, mode):
        self.log("ConfirmModeChange (%s)" % (mode))

    @dbus.service.method("org.bluez.Agent",
            in_signature="", out_signature="")
    def Cancel(self):
        self.log("Cancel")

    @dbus.service.method(dbus_interface="net.aircable.airi.Pair",
            in_signature='', out_signature='b')
    def isRegistered(self):
        return len(Agent.listeners)>0


def registerControlSignals(bus):
    bus.add_signal_receiver(handle_name_owner_changed,
        'NameOwnerChanged',
        'org.freedesktop.DBus',
        'org.freedesktop.DBus',
        '/org/freedesktop/DBus')

    bus.add_signal_receiver(
            partial(handle_adapter_added, bus=bus),
            signal_name='AdapterAdded',
            dbus_interface='org.bluez.Manager',
            member_keyword='signal')

    bus.add_signal_receiver(handle_adapter_removed,
            signal_name='AdapterRemoved',
            dbus_interface='org.bluez.Manager',
            member_keyword='signal')

def initAgent(bus):
    try:
        manager = dbus.Interface(bus.get_object("org.bluez", "/"),
                "org.bluez.Manager")
        f = True
        for path in manager.ListAdapters():
            if not registerAgent(bus, path):
                f=False
        if f:
            log.msg("Agent registered on all paths")
    except Exception, err:
        log.err("Something went wrong on the agent application")
        log.err(err)

def main():
    try:
        bus = dbus.SessionBus()
        manager = dbus.Interface(bus.get_object("org.bluez", "/"),
                "org.bluez.Manager")
        log.msg("Using session bus")
    except Exception, err:
        print err
        bus = dbus.SystemBus()
        log.msg("Using system bus")
    registerControlSignals(bus)
    agent = Agent(bus, PATH)
    agent.set_exit_on_release(False)
    initAgent(bus)

if __name__ == '__main__':
    log.startLogging(sys.stdout)
    reactor.callWhenRunning(main)
    reactor.run()
    log.msg("Agent is exiting")




# -*- coding: utf-8 -*-
# Copyright (c) 2011 Naranjo Manuel Francisco <manuel@aircable.net>
# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
# See Twisted LICENSE for details.

"""
Various Bluetooth Socket classes
"""

# System Imports
import os
import types
import socket
import sys
import operator
import bluetooth
import struct

from twisted.internet import tcp, reactor
from twisted.internet.tcp import *

from errno import EBADFD

from twisted.python import log

from bluetooth import *
import bluetooth._bluetooth as bt

SPP_UUID="00001101-0000-1000-8000-00805F9B34FB"
if __name__=='__main__':
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)

L2CAP_SUPPORTED=True
SCO_SUPPORTED=True

flag = False
def initalizeDBus():
    global flag
    if flag:
        return

    global dbus, bus, manager
    try:
        import dbus
        try:
            bus = dbus.SessionBus()
            manager = dbus.Interface(bus.get_object("org.bluez", "/"),
                "org.bluez.Manager")
        except:
            bus = dbus.SystemBus()
            manager = dbus.Interface(bus.get_object("org.bluez", "/"),
                "org.bluez.Manager")
        log.msg("DBus available")
        flag = True
    except Exception, err:
        log.msg("DBus unavailable")
        log.error(err)
        if isAndroid(): flag = True

def isAndroid():
    try:
        global droid
        import android
        droid=android.API()
        return True
    except:
        pass
    return False

if isAndroid():
    PAIR_SUPPORTED=False
else:
    PAIR_SUPPORTED=True

def androidIsBonded(address):
    try:
        global droid
        r = droid.bluetoothIsBonded(address)
        return r
    except Exception, err:
        log.err(err)
    return False

def bluezIsBonded(address):
    initalizeDBus()
    global dbus, bus, manager
    
    adapter = dbus.Interface(bus.get_object("org.bluez",
        manager.DefaultAdapter()), "org.bluez.Adapter")
    try:
        dev = adapter.FindDevice(address)
    except Exception, err:
        return False
    dev = dbus.Interface(bus.get_object("org.bluez",dev),
            "org.bluez.Device")
    return bool(dev.GetProperties()["Paired"])
    
def isBonded(address):
    if isAndroid():
        return androidIsBonded(address)
    return bluezIsBonded(address)

def androidBondDevice(address):
    res = droid.bluetoothConnect(SPP_UUID, address) 
    droid.bluetoothStop(res)

def bluezBondDevice(address):
    initalizeDBus()
    global dbus, bus, manager
    
    from airi.pair import Agent, PATH
    path = "%s/temp/%s" % (PATH, address.replace(":", ""))
    agent = Agent(bus, path)
    agent.set_exit_on_release(False)
    adapter = dbus.Interface(bus.get_object("org.bluez",
        manager.DefaultAdapter()), "org.bluez.Adapter")
    log.msg("Bonding device")
    adapter.CreatePairedDevice(address, path,
            "KeyboardOnly", reply_handler=lambda X: None,
            error_handler=lambda X: None)

def bondDevice(address):
    if isAndroid():
        return androidBondDevice(address)
    return bluezBondDevice(address)

def isPairingSupported():
    if isAndroid():
        return False
    print "isPairingSupported", True
    return True

def isPairingReady():
    if not isPairingSupported():
        return False
    return len(Agent.listeners)>0

class BluetoothConnection(tcp.Connection):
    """
    Superclass of all Bluetooth-socket-based Descriptors

    This is an abstract superclass of all objects which represent a Bluetooth
    connection based socket.

    @ivar logstr: prefix used when logging events related to this connection.
    @type logstr: C{str}

    """

class BluetoothBaseClient(tcp.Connection):
    """A base class for client Bluetooth (and similiar) sockets.
    """
    addressFamily = 31 # AF_BLUETOOTH
    proto = None

    def createInternetSocket(self):
        """(internal) Create a non-blocking socket using
        self.addressFamily, self.socketType.
        """
        if self.proto not in [ None, bluetooth.RFCOMM, bluetooth.SCO, 
                bluetooth.HCI, bluetooth.L2CAP ]:
            raise RuntimeException("I only handle bluetooth sockets")

        print self.proto, type(self.proto)
        s = bluetooth.BluetoothSocket(self.proto)
        s.setblocking(0)
        tcp.fdesc._setCloseOnExec(s.fileno())
        return s
    
    def resolveAddress(self):
        self.realAddress = self.addr
        print "resolveAddress", self.realAddress
        self.doConnect()

    def doConnect(self):
        """I connect the socket.

        Then, call the protocol's makeConnection, and start waiting for data.
        """
        if not hasattr(self, "connector"):
            # this happens when connection failed but doConnect
            # was scheduled via a callLater in self._finishInit
            return

        err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if err:
            self.failIfNotConnected(error.getConnectError((err, strerror(err))))
            return

        # doConnect gets called twice.  The first time we actually need to
        # start the connection attempt.  The second time we don't really
        # want to (SO_ERROR above will have taken care of any errors, and if
        # it reported none, the mere fact that doConnect was called again is
        # sufficient to indicate that the connection has succeeded), but it
        # is not /particularly/ detrimental to do so.  This should get
        # cleaned up some day, though.
        try:
            connectResult = self.socket.connect_ex(self.realAddress)
        except socket.error, se:
            connectResult = se.args[0]
        print "connectResult", connectResult
        if connectResult:
            if connectResult == EISCONN or connectResult == EBADFD:
                pass 
            elif connectResult in (EWOULDBLOCK, EINPROGRESS, EALREADY):
                self.startReading()
                self.startWriting()
                return
            else:
                self.failIfNotConnected(error.getConnectError((connectResult, 
                    strerror(connectResult))))
                return

        # If I have reached this point without raising or returning, that means
        # that the socket is connected.
        del self.doWrite
        del self.doRead
        # we first stop and then start, to reset references to the old doRead
        self.stopReading()
        self.stopWriting()
        self._connectDone()

    def doRead(self):
        """Calls self.protocol.dataReceived with all available data.

        This reads up to self.bufferSize bytes of data from its socket, then
        calls self.dataReceived(data) to process it.  If the connection is not
        lost through an error in the physical recv(), this function will return
        the result of the dataReceived call.
        """
        try:
            data = self.socket.recv(self.bufferSize)
        except (bluetooth.btcommon.BluetoothError, socket.error), se:
            if se.args[0] == EWOULDBLOCK:
                return
            else:
                return main.CONNECTION_LOST
        if not data:
            return main.CONNECTION_DONE
        return self.protocol.dataReceived(data)



class Client(BluetoothBaseClient, tcp.Client):
    """A Bluetooth client."""

    def __init__(self, proto, *args, **kwargs):
        if proto not in [ None, bluetooth.RFCOMM, bluetooth.SCO, bluetooth.HCI, 
                bluetooth.L2CAP ]:
            raise RuntimeException("I only handle bluetooth sockets")
        self.proto = proto
        super(Client, self).__init__(*args, **kwargs)

    def getHost(self):
        """Returns a Bluetooth address.

        This indicates the address from which I am connecting.
        """
        return self.socket.getsockname()

    def getPeer(self):
        """Returns a Bluetooth address.

        This indicates the address that I am connected to.
        """
        return self.realAddress

class Server(BluetoothBaseClient, tcp.Connection):
    """
    Serverside bluetooth socket-stream connection class.

    This is a serverside network connection transport; a socket which came from
    an accept() on a server.
    """

    def startTLS(self, ctx, server=1):
        raise RuntimeException("not valid method")

    def getHost(self):
        """Returns a Bluetooth address.

        This indicates the server's address.
        """
        return self.socket.getsockname()

    def getPeer(self):
        """Returns a Bluetooth address.

        This indicates the client's address.
        """
        return self.client

class Port(tcp.Port):
    """
    A Bluetooth server port, listening for connections.

    L{twisted.internet.tcp.Port}
    """
    addressFamily = 31 # socket.AF_BLUETOOTH
    proto = None

    def __init__(self, proto, port, factory, backlog=50, interface='', 
            reactor=None):
        """Initialize with a numeric port to listen on.
        """
        tcp.Port.__init__(self, port, factory, backlog, interface, reactor)
        if proto not in [ None, bluetooth.RFCOMM, bluetooth.SCO, bluetooth.HCI, 
                bluetooth.L2CAP ]:
            raise RuntimeException("I only do Bluetooth")
        self.proto = proto

    def createInternetSocket(self):
        s = bluetooth.BluetoothSocket(self.proto)
        if platformType == "posix" and sys.platform != "cygwin":
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s

    def _buildAddr(self, (host, port)):
        return (host, port)

    def getHost(self):
        """Returns a Bluetooth Address

        This indicates the server's address.
        """
        return self.socket.getsockname()

class Connector(tcp.Connector):
    def __init__(self, proto, host, port, *a, **kw):
        tcp.Connector.__init__(self, host, int(port), *a, **kw)
        self.proto = proto

    def _makeTransport(self):
        return Client(self.proto, self.host, self.port, self.bindAddress, self, 
                self.reactor)

    def getDestination(self):
        return (self.host, self.port)

def __connectGeneric(reactor, proto, host, port, factory, timeout=30, 
        bindAddress=None):
    if not isBonded(host) and proto != bluetooth.SCO:
        bondDevice(host)
    c = Connector(proto, host, port, factory, timeout, bindAddress, reactor)
    c.connect()
    return c

def connectRFCOMM(reactor, *a, **kw):
    return __connectGeneric(reactor, bluetooth.RFCOMM, *a, **kw)

def connectL2CAP(reactor, *a, **kw):
    return __connectGeneric(reactor, bluetooth.L2CAP, *a, **kw)

def setupSCO():
    # check voice settings. switch to S16 LE mono 8kHz CVSD if needed
    hci_sock=bt.hci_open_dev()
    response = bt.hci_send_req(hci_sock, bt.OGF_HOST_CTL, 
            bt.OCF_READ_VOICE_SETTING, bt.EVT_CMD_COMPLETE, 3)
    status, voice_setting = struct.unpack("<BH", response)
    if voice_setting != 0x60:
        new_vs = struct.pack("<H", 0x60)
        bt.hci_send_req(hci_sock, bt.OGF_HOST_CTL, 
            bt.OCF_WRITE_VOICE_SETTING, 
            bt.EVT_CMD_COMPLETE, 1, new_vs)

    # determine the maximun packet size
    response = bt.hci_send_req(hci_sock, bt.OGF_INFO_PARAM, 
    bt.OCF_READ_BUFFER_SIZE, bt.EVT_CMD_COMPLETE, 8)
    status, acl_mtu, sco_mtu, acl_nbufs, sco_nbufs = struct.unpack(
                ">BHBHH", response)
    return acl_mtu, sco_mtu, acl_nbufs, sco_nbufs
acl_mtu, sco_mtu, acl_nbufs, sco_nbufs = setupSCO()

def connectSCO(reactor, *a, **kw):
    return __connectGeneric(reactor, bluetooth.SCO, *a, **kw)

def resolve_name(address):
    sock = bluetooth.bluez._gethcisock()
    out = None
    try:
        out = bluetooth.bluez._bt.hci_read_remote_name(sock, address)
    except:
        pass
    sock.close()
    return out

def SCOConnected(target):
    return target.lower() in SCOReader.clients

def SCODisconnect(target):
    if not SCOConnected(target):
        return
    c = SCOReader.clients[target.lower()]
    c.close()

class SCOReader(object):
    sock = None
    listener = None
    clients = {}

    def __init__(self, target, listener = None, connectionLost = None):
        if SCOConnected(target):
            log.error("Can't connect twice to the same device")
            return

        self.sock = bluetooth.BluetoothSocket(bluetooth.SCO)
        try:
            self.sock.connect((target,))
        except Exception, err:
            log.error("Failed while trying to connect SCO")
            log.error(err)
            return
        self.sock.setblocking(False)
        self.address = target.lower()
        self._listener = listener
        self._connectionLost = connectionLost
        SCOReader.clients[target.lower()]=self
        reactor.addReader(self)

    def isConnected(self):
        return self.sock != None

    def fileno(self):
        return self.sock.fileno()

    def doRead(self):
        self._listener(self)

    def close(self):
        reactor.removeReader(self)
        self.sock.setblocking(True)
        self.sock.close()
        self.sock = None
        if self.address.lower() in SCOReader.clients:
            del SCOReader.clients[self.address.lower()]

    def logPrefix(self): 
        return "SCO-Reader"

    def connectionLost(self, reason):
        if callable(self._connectionLost):
            try:
                self._connectionLost(self)
            except Exception, err:
                log.error(err)
        if self.address.lower() in SCOReader.clients:
            del SCOReader.clients[self.address.lower()]
    
    def recv(self, *args, **kwargs):
        return self.sock.recv(*args, **kwargs)

if __name__=='__main__':
    import sys
    address = sys.argv[1]
    import airi.pair as pair
    def main():
        initalizeDBus()
        global dbus, bus, manager
        agent = pair.Agent(bus, pair.PATH)
        print isBonded(address)
        print bondDevice(address)
        print isBonded(address)
        reactor.stop()
    from twisted.internet import reactor
    reactor.callWhenRunning(main)
    reactor.run()




# -*- coding: utf-8 -*-
import os, sys
from setuptools import setup, find_packages
from airi import __version__


def read(fname):
    '''
    Utility function to read the README file.
    Used for the long_description.  It's nice, because now 1) we have a top
    level README file and 2) it's easier to type in the README file than to put
    a raw string in below
    '''
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

REQUIRES = [
    "Twisted >= 10.0.0",
    "Jinja2"
]

if sys.platform.startswith("linux") or sys.platform.startswith("win"):
    REQUIRES.append("PyBluez >= 0.18")
elif sys.platform.startswith("darwin"):
    REQUIRES.append("lightblue >=0.4.1")

setup(name="airi",
    version=__version__,
    author="Naranjo Manuel Francisco",
    author_email="manuel@aircable.net",
    description=("AIRi software package"),
    license="Apache V2",
    keywords="airi bluetooth twisted",
    url="https://github.com/manuelnaranjo/AIRi",
    packages=find_packages("."),
    long_description=read('README'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Framework :: Twisted",
        "Topic :: Communications",
        "Topic :: Multimedia :: Video :: Capture",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
    ],
    include_package_data=True,
    package_data={
      'airi': ['airi/media/*.*', 'airi/templates/*.*'],
    },
    #scripts=["AIRi"],
    entry_points = {
      'console_scripts': [
        'AIRi = airi.main:main', 
        'AIRi-setup = airi.install:main'
      ]
    },
    dependency_links = [
	"http://code.google.com/p/pybluez/downloads/list",
	"https://github.com/manuelnaranjo/LightBlue---Mac/zipball/release1#egg=lightblue-0.4.1"
    ],
    install_requires = REQUIRES,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from twisted.internet.protocol import Protocol, ClientFactory
import airi.twisted_bluetooth as twisted_bluetooth
from twisted.python import log
from airi.camera import dbg, Listener, UnknownDevice
from twisted.internet import reactor
import optieyes
import airicamera
from functools import partial
from airi.settings import getSettings
from airi import report
from time import localtime, strftime
settings = getSettings()


CATEGORY = "airi.camera.protocol"

TYPES = {
    'OPTIEYE': {
        "welcome": "genie cam on",
        "class": optieyes.OptiEye},
    'AIRI': {
        "welcome": "$airi",
        "class": airicamera.AIRi},
}


def getTagByType(tag):
    out = {}
    for k in TYPES:
        out[k] = TYPES[k][tag]
    return out

WELCOME = getTagByType("welcome")

INIT = 0
IDLE = 1


class Camera(Protocol):
    kind = None
    state = INIT
    callLater = None
    transport = None

    def __init__(self, address):
        dbg("new camera instance for (%s)", address)
        setattr(self, "client", None)
        setattr(self, "buffer", "")
        self.address = address

    def invalidCamera(self):
        log.msg("not known camera, we stop")
        self.transport.loseConnection()

    def doINIT(self):
        def welcomeCheck():
            while len(self.buffer) > 0 and self.buffer.find('\n') > -1:
                welcome, self.buffer = self.buffer.split('\n', 1)
                dbg("welcome", welcome)
                welcome = welcome.strip().lower().strip('\x00')
                for k, v in WELCOME.iteritems():
                    if welcome.find(v) > -1:
                        return k

        dbg("doINIT", self.buffer)
        if self.buffer.find("\n") == -1:
            dbg("Welcome not received")
            if not self.callLater:
                self.callLater = reactor.callLater(3, self.invalidCamera)
            self.callLater.reset(2)
            return

        kind = welcomeCheck()
        if not kind:
            if not self.callLater:
                self.callLater = reactor.callLater(3, self.invalidCamera)
            self.callLater.reset(2)
            return

        self.kind = kind
        log.msg("Connected to a %s camera" % self.kind)
        dbg("still in buffer", len(self.buffer))
        self.state = IDLE
        settings.setCameraSetting(self.address, "address", self.address)
        settings.setCameraSetting(self.address, "type", self.kind)
        settings.setCameraSetting(self.address, "name", "NN")
        settings.setCameraSetting(self.address, "last",
                                    strftime("%m/%d/%Y %H:%M:%S", localtime()))
        settings.save()
        self.client = TYPES[self.kind]["class"](self)
        if self.callLater:
            self.callLater.cancel()
            self.callLater = None

    def dataReceived(self, data):
        self.buffer += data
        if self.state == INIT:
            return self.doINIT()
        elif self.client:
            return self.client.dataReceived()

    def __getattr__(self, val):
        if val in [
              'getType', 'getCapabilities', 'getSize', 'getFlash',
              'getBattery', 'getTransport', 'getExposure', 'getZoom', 'getPan',
              'getVoice', 'updateSettings']:
            return getattr(self.client, val, lambda: False)
        raise AttributeError


class CameraFactory(ClientFactory):
    FACTORY = None
    clients = {}
    listeners = {}
    pending = []

    @report(category=CATEGORY)
    def startedConnecting(self, connector):
        pass

    @report(category=CATEGORY)
    def buildProtocol(self, addr):
        log.msg("Connected %s" % str(addr))
        if addr in CameraFactory.pending:
            CameraFactory.pending.remove(addr)
        from airi.api import UpdateManager
        UpdateManager.propagate(addr[0], {"status": True})
        CameraFactory.clients[addr[0]] = Camera(addr[0])
        CameraFactory.clients[addr[0]].gotFrame = partial(
            CameraFactory.gotFrame, addr[0])
        return CameraFactory.clients[addr[0]]

    #@report(category=CATEGORY)
    @classmethod
    def gotFrame(klass, addr, frame):
        if addr not in klass.listeners:
            return
        for listener in klass.listeners[addr]:
            listener.gotFrame(frame, address=addr)

    @classmethod
    @report(category=CATEGORY)
    def setSize(klass, addr, size):
        klass.clients[addr].client.setSize(size)

    @classmethod
    @report(category=CATEGORY)
    def getSize(klass, addr):
        return klass.clients[addr].client.getSize()

    @classmethod
    @report(category=CATEGORY)
    def getSizes(klass, addr):
        return klass.clients[addr].client.getSizes()

    @classmethod
    @report(category=CATEGORY)
    def __cleanup(klass, addr):
        if addr in klass.listeners:
            del klass.listeners[addr]
        if addr in klass.clients:
            if getattr(klass.clients[addr], "client", None):
                del klass.clients[addr].client
            del klass.clients[addr]

    @report(category=CATEGORY)
    def __lostConnection(self, addr, reason, failed):
        rep = False

        if addr in CameraFactory.pending:
            CameraFactory.pending.remove(addr)

        if addr in CameraFactory.listeners:
            for listener in CameraFactory.listeners[addr]:
                rep = rep or listener.lostConnection(reason, failed,
                                    address=addr)

        if not rep:
            CameraFactory.__cleanup(addr)
        from airi.api import UpdateManager
        UpdateManager.propagate(addr, {"status": False})

        if __name__ == '__main__' and reactor.running:
            reactor.stop()

    @report(category=CATEGORY)
    def clientConnectionLost(self, connector, reason):
        addr = connector.getDestination()[0]
        log.msg("Lost connection to %s. Reason: %s" % (addr, reason))
        self.__lostConnection(addr, reason, False)

    @report(category=CATEGORY)
    def clientConnectionFailed(self, connector, reason):
        addr = connector.getDestination()[0]
        log.msg("Connection Failed to %s. Reason: %s" % (addr, reason))
        self.__lostConnection(addr, reason, True)

    @classmethod
    @report(category=CATEGORY)
    def isConnected(klass, addr):
        return addr in klass.clients

    @classmethod
    @report(category=CATEGORY)
    def isPending(klass, addr):
        return addr in klass.pending

    @classmethod
    @report(category=CATEGORY)
    def connect(klass, address, channel=1, method="RFCOMM"):
        if klass.isConnected(address):
            raise Exception("All ready connected to %s" % address)

        if klass.isPending(address):
            raise Exception("Connection to %s is pending" % address)

        klass.pending.append(address)
        settings = klass.getCamera(address, True)
        method = getattr(settings, "transport", method)
        c = getattr(twisted_bluetooth, "connect%s" % method)

        if klass.FACTORY == None:
            klass.FACTORY = klass()

        if method == "RFCOMM":
            channel = 1
        elif method == "L2CAP":
            channel = 0x1001
        log.msg("Connecting to %s using %s channel %s" %
                (address, method, channel))

        from airi.api import UpdateManager
        UpdateManager.propagate(address, {"status": "Connecting"})
        c(reactor, address, channel, klass.FACTORY)

    @classmethod
    @report(category=CATEGORY)
    def disconnect(klass, address):
        if klass.isConnected(address):
            try:
                klass.clients[address].client.disconnect()
                klass.clients[address].transport.loseConnection()
            except Exception, err:
                log.msg("call to disconnect failed")
                log.err(err)
        if address in klass.listeners:
            for listener in klass.listeners[address]:
                if getattr(listener, 'forcedDisconnect', None):
                    try:
                        listener.forcedDisconnect(address=address)
                    except:
                        pass
        klass.__cleanup(address)

    @classmethod
    @report(category=CATEGORY)
    def registerListener(klass, address, listener):
        if not address in klass.listeners:
            klass.listeners[address] = list()

        if listener not in klass.listeners[address]:
            dbg("adding listener")
            klass.listeners[address].append(listener)

    @classmethod
    @report(category=CATEGORY)
    def removeListener(klass, address, listener):
        try:
            klass.listeners[address].remove(listener)
        except:
            pass

    @classmethod
    @report(category=CATEGORY)
    def getConnected(klass, address=None):
        if address:
            return klass.clients.get(str(address.strip()), None)
        return klass.clients

    @classmethod
    @report(category=CATEGORY)
    def getCamera(klass, address, silent=False):
        out = settings.getCamera(address)
        if not out:
            if not silent:
                raise UnknownDevice(address)
            return None

        out["status"] = klass.isConnected(address)
        if klass.isConnected(address):
            out["name"] = twisted_bluetooth.resolve_name(address)
            settings.setCamera(out)
            settings.save()
        out["capabilities"] = TYPES[out["type"]]["class"].Capabilities
        return out

    @classmethod
    @report(category=CATEGORY)
    def getCameras(klass):
        for camera in settings.getCameras():
            c = klass.getCamera(camera["address"], silent=True)
            if c: yield c

    @classmethod
    def getTypes(klass):
        return TYPES.keys()


if __name__ == '__main__':
    import sys

    class TestListener(Listener):

        def lostConnection(self, reason, failed):
            return False

        def gotFrame(self, frame, *a, **kw):
            print "got frame %s bytes long" % len(frame)
            CameraFactory.disconnect(sys.argv[1])

    if len(sys.argv) != 3:
        print "Usage %s <target> <channel>" % sys.argv[0]
        sys.exit(1)

    log.startLogging(sys.stdout)
    CameraFactory.connect(sys.argv[1], int(sys.argv[2]))
    CameraFactory.registerListener(sys.argv[1], TestListener())
    reactor.run()

